@@ -18,7 +18,7 @@ import f90nml
 class database:
     """This class defines an object that accesses density files stored online and index in pandas table."""
     
-    def  __init__(self,webbase="https://datapub.fz-juelich.de/anogga/files/densindx/",pddf_hdf="densities_table.h5",workdir=""):
+    def  __init__(self,webbase="https://just-object.fz-juelich.de:8080/v1/AUTH_1715b19bd3304fb4bb04f4beccea0cf2/densitystore/",pddf_hdf="densities_table.h5",workdir=""):
         """Initializes the object that gives read and write access to density files
            - webbase url of online storage for files
              in most cases, files will be already uploaded. Uploading is possible using methods defined below. 
@@ -55,18 +55,21 @@ class database:
 
         print("table_online:",table_online)
 
-        # download table if online available
-        # or generate empty one if neither online nor offline available
-
-        if table_online:
-           self._download_file_(url,workdir)    
-           print("Table file is downloaded: ",url)
-        
         # check whether table is locally available
 
         table_local=os.path.exists(workdir+pddf_hdf)
 
         print("table_local:",table_local)
+
+        # use local file if available 
+        # download table if online available
+        # or generate empty one if neither online nor offline available
+
+        if table_online and not table_local:
+           self._download_file_(url,workdir)    
+           print("Table file is downloaded: ",url)
+           table_local=True 
+
         
         # if file could not be downloaded and is not available
         # create an empty table otherwise load  table
@@ -104,63 +107,16 @@ class database:
         return local_filename
 
 
-    def add_file(self,filename=None,kind="one", 
-                 nninter="sms",order="lo",cutoff=450,
-                 tnf="None",tnfcut=450,c1=0.0,c3=0.0,c4=0.0,ce=0.0,cd=0.0,cE1=0.0,cE2=0.0,cE3=0.0,cE4=0.0,cE5=0.0,cE6=0.0,cE7=0.0,cE8=0.0,cE9=0.0,cE10=0.0,
-                 srg="None",srgval=0.0,
-                 N=1,Z=2,Jtot=1,MTtot=1,
-                 np12a=0,np12b=0,
-                 j12max=5,lmax=6,Tmax=3,lsummax=0,
-                 Nmax=0,OmegaHO=0.0,
-                 omega=0.0,theta=0.0,**kwargs
-               ):
+    def add_file(self,filename=None,**labels):
         """This routine adds a file to the data base locally
            filename - absolute path to local file
-           kind - "one" or "two" for one- or two-body density, 
-           nninter - nn interaction used,
-           order - chiral order of interaction "lo,nlo,n2lo,...,n4lo+",
-           cutoff - numerical value of cutoff (either momentum or rspace),
-           tnf - 3NF used "None", "n2lo","n3lo",
-           tnfcut - cutoff for the 3NF, 
-           c1,c3,c4 - N2LO parameters of 3NF,
-           cd,ce   - N2LO contact parameters of 3NF   
-           cE1,...,cE10 - N4LO parameters of 3NF, 
-           srg - "None","2body","3body", 
-           srgval - if srg="2body" or "3body", value used for the SRG evolution (s or cutoff),
-           N - number of neutrons in nucleus,
-           Z - number of protons in nucleus,
-           Jtot  - twice total angular momentum of state,
-           MTtot - twice third component of isospin,
-           np12a,np12b - number of grid points, 
-           j12max - max. j12 used for calculation,
-           lmax - max. orbital angular momentum used in calc,
-           lsummax - max. sum of orbital angular momentum used in calc,
-           Tmax - twice maximal isospin used in calculation,
-           Nmax - maximal HO quantum number in NCSM if applicable,
-           OmegaHO - HO frequency used in NCSM calculation if applicable
-           omega - incoming photon energy (for compton scattering)
-           theta - photon scattering angle in deg
+           labels should be given according to standard set (see in _prep_label_).
+           returns hashname of the file
         """
 
         # prepare dictionary specifying the case wanted
-
-        specs={"kind":kind.lower(), 
-               "nninter":nninter.lower(),
-               "order":order.lower(),
-               "cutoff":cutoff,
-               "tnf":tnf.lower(),"tnfcut":tnfcut,
-               "c1":c1,"c3":c3,"c4":c4,"cd":cd,"ce":ce,
-               "cE1":cE1,"cE2":cE2,"cE3":cE3,"cE4":cE4,"cE5":cE5,"cE6":cE6,"cE7":cE7,"cE8":cE8,"cE9":cE9,"cE10":cE10,
-               "srg":srg.lower(),"srgval":srgval,
-               "N":N,"Z":Z,"Jtot":Jtot,"MTtot":MTtot,
-               "j12max":j12max,"lmax":lmax,"lsummax":lsummax,"Tmax":Tmax,
-               "np12a":np12a,"np12b":np12b,
-               "Nmax":Nmax,"OmegaHO":OmegaHO,
-               "omega":omega,"theta":theta}
-
-
-        # check specs for allowed values
-        self._check_specs_(specs)
+        # also does some basic checks of consistency 
+        specs=self._prep_label_(**labels)
 
         # prepare unique file name for internal use
 
@@ -219,14 +175,207 @@ class database:
         # returns hashed name of the file
         return hashname 
 
+    def _prep_label_(self,**labels):
+            """This method augments the labels by a standard set of labels given as
+               shown in source code. Some basic checks of the file labels are performed. 
+            """
+            standard_labels={'Z':0,'N':0,"E[MeV]":0.0,
+              'NP12A':0, 'NP12B':0, 'NP12':0, 'P12A':0, 'P12B':0,'P12C':0,
+              'NP34A':0, 'NP34B':0, 'NP34':0, 'P34A':0, 'P34B':0,'P34C':0,
+              'NP3A':0, 'NP3B':0, 'NP3':0, 'P3A':0, 'P3B':0, 'P3C':0,
+              'NQ4A':0, 'NQ4B':0, 'NQ4':0, 'Q4A':0, 'Q4B':0, 'Q4C':0,
+              'NQA':0, 'NQB':0, 'NQ':0, 'QA':0, 'QB':0, 'QC':0,
+              'NPTA':0, 'NPTB':0,'NPT':0, 'PTA':0, 'PTB':0, 'PTC':0,
+              'NEA':0, 'NEB':0, 'NE':0, 'EA':0, 'EB':0, 'EC':0,
+              'NX':0,
+              'taumax':0, 'lsummax':0,'lmax':0, 'j12max':0, 'l3max':0,'l4max':0,'lammax':0,'MTtot':0, 'Jtot':0,
+              'MODENN':'none', 'orderNN':'none','LambdaNN':0.0, 'potnr':0, 'empotnr':0,
+              'lam3NF':0.0, 'c1':0.0, 'c3':0.0, 'c4':0.0, 'cd':0.0, 'ce':0.0,
+              'cE1':0.0, 'cE2':0.0, 'cE3':0.0, 'cE4':0.0, 'cE5':0.0, 'cE6':0.0, 'cE7':0.0, 'cE8':0.0, 'cE9':0.0, 'cE10':0.0,
+              'tnforder':'no3nf','j12max3nf':0,'j3max3nf':0,
+              'relcalc':0,
+              'nnsrg':0, 'tnfsrg':0, 'lambdaSRGNN':0.0,'lambdaSRG3N':0.0, 'potbareNN':0, 'potbareNNTNF':0, 'ostatSRGNN':0, 'ostatSRGTNF':0,
+              'cutnumSRGNN':0.0, 'cutnumSRGTNF':0.0,'omega':0.0, 'theta':0.0,
+              'j12maxdens':0, 'taumaxdens':0, 'lmaxdens':0, 'nphitilde':0, 'nxtilde':0, }
+        
+            out_label={**standard_labels,**labels}
+
+            # remove entries for uniquefilename,localonly,localavail,addtime,moditime
+            try:
+              del out_label['uniquefilename']
+            except:
+              pass  
+            try:
+              del out_label['localonly']
+            except:
+              pass  
+            try:
+              del out_label['localavail']
+            except:
+              pass  
+            try:
+              del out_label['addtime']
+            except:
+              pass  
+            try:
+              del out_label['moditime']
+            except:
+              pass  
+
+            try:
+              out_label["MTtot"]=out_label["mt4"] 
+            except:
+              pass  
+            try:
+              out_label["Jtot"]=out_label["j4"]  
+            except:
+              pass  
+          # remove out_label to shorten dataset, keep energy 
+            try:
+              del out_label["H1[MeV]"]
+            except:
+              pass  
+            try:
+              del out_label["H2[MeV]"]
+            except:
+              pass  
+            try:
+              del out_label["ECSB[MeV]"]
+            except:
+              pass  
+            try:
+              del out_label["T[MeV]"]
+            except:
+              pass  
+            try:
+              del out_label["VNN1[MeV]"]
+            except:
+              pass  
+            try:
+              del out_label["VNN2[MeV]"]
+            except:
+              pass  
+            try:
+              del out_label["V3N[MeV]"]
+            except:
+              pass  
+            try:
+              del out_label["CSB[keV]"]
+            except:
+              pass  
+            try:
+              del out_label["NORM31"]
+            except:
+              pass  
+            try:
+              del out_label["NORM22"]
+            except:
+              pass  
+            try:
+              del out_label["P(S)[%]"]
+            except:
+              pass  
+            try:
+              del out_label["P(P)[%]"]
+            except:
+              pass  
+            try:
+              del out_label["P(D)[%]"]
+            except:
+              pass  
+            try:
+              del out_label["P(p)[%]"]
+            except:
+              pass  
+            try:
+              del out_label["P(n)[%]"]
+            except:
+              pass  
+            try:
+              del out_label["P(pp)[%]"]
+            except:
+              pass  
+            try:
+              del out_label["P(np)[%]"]
+            except:
+              pass  
+            try:
+              del out_label["P(nn)[%]"]
+            except:
+              pass  
+            try:
+              del out_label["r(p)[fm]"]
+            except:
+              pass  
+            try:
+              del out_label["r(n)[fm]"]
+            except:
+              pass  
+            try:
+              del out_label["r(NN)[fm]"]
+            except:
+              pass  
+            try:
+              del out_label["ETA"]
+            except:
+              pass  
+            try:
+              del out_label["DEV"]
+            except:
+              pass  
+            try:
+              del out_label["mt4"]
+            except:
+              pass  
+            try:
+              del out_label["j4"]   
+            except:
+              pass  
+          
+       
+            # check the grid point numbers to find sums, if necessary 
+            if out_label['NP12A']+out_label['NP12B']!=0:
+               out_label['NP12']=out_label['NP12A']+out_label['NP12B']   
+            if out_label['NP3A']+out_label['NP3B']!=0:
+               out_label['NP3']=out_label['NP3A']+out_label['NP3B']   
+            if out_label['NQ4A']+out_label['NQ4B']!=0:
+               out_label['NQ4']=out_label['NQ4A']+out_label['NQ4B']   
+            if out_label['NQA']+out_label['NQB']!=0:
+               out_label['NQ']=out_label['NQA']+out_label['NQB']   
+            if out_label['NP34A']+out_label['NP34B']!=0:
+               out_label['NP34']=out_label['NP34A']+out_label['NP34B']   
+            if out_label['NPTA']+out_label['NPTB']!=0:
+               out_label['NPT']=out_label['NPTA']+out_label['NPTB']         
+            if out_label['NEA']+out_label['NEB']!=0:
+               out_label['NE']=out_label['NEA']+out_label['NEB']         
+
+            # make all str lower characters 
+            for key in out_label.keys():    
+              if type(out_label[key])==type('test'):
+                 out_label[key]=out_label[key].lower()
+
+            if not (out_label["MODENN"] in ["av18","chsms","cdbonn","ibm189"]):
+               raise(ValueError("'MODENN' not defined"))
+            if not (out_label["tnforder"] in ["no3nf","n2lo"]):
+               raise(ValueError("'tnforder' not defined"))
+            if (out_label["N"] != int(out_label["N"])) or (out_label["N"] < 1) :
+               raise(ValueError("'N' not defined"))
+            if (out_label["Z"] != int(out_label["Z"])) or (out_label["Z"] < 1) :
+               raise(ValueError("'Z' not defined"))
+            if (out_label["Jtot"] != int(out_label["Jtot"])) or (out_label["Jtot"] < 0) :
+               raise(ValueError("'Jtot' not defined"))
+            if  out_label["MTtot"] != int(out_label["MTtot"]) :
+               raise(ValueError("'MTtot' not defined"))
+           
+            return out_label        
 
     def _unique_filename_(self,specs):
         """prepares a unique file name and its hash value"""
 
-        uniquefilename="comp-dens-Z={Z}-N={N}-{kind}-{nninter}-{order}-cut={cutoff}".format(**specs)
+        uniquefilename="comp-dens-{kind}-Z={Z}-N={N}-mt={MTtot}-J={Jtot}-{MODENN}-{orderNN}-cut={LambdaNN}".format(**specs)
 
-        if specs["tnf"] != "none": 
-          uniquefilename+="-tnf={tnf}-tnfcut={tnfcut}-c1={c1}-c3={c3}-c4={c4}-cd={cd}-ce={ce}".format(**specs)
+        if specs["tnforder"] != "no3nf": 
+          uniquefilename+="-tnf={tnforder}-tnfcut={lam3NF}-c1={c1}-c3={c3}-c4={c4}-cd={cd}-ce={ce}".format(**specs)
           if specs["cE1"] != 0.0:  
             uniquefilename+="-cE1={cE1}".format(**specs)
           if specs["cE2"] != 0.0:  
@@ -247,120 +396,77 @@ class database:
             uniquefilename+="-cE9={cE9}".format(**specs)
           if specs["cE10"] != 0.0:  
             uniquefilename+="-cE10={cE10}".format(**specs)
-        if specs["srg"] != "none":
-          uniquefilename+="-srg={srg}={srgval}".format(**specs)
-        if specs["OmegaHO"] != 0.0:
-          uniquefilename+="-Nmax={Nmax}-Om={OmegaHO}".format(**specs)
-        uniquefilename+="-np12a={np12a}-np12b={np12b}".format(**specs)
-        uniquefilename+="-jmx={j12max}-lmx={lmax}-lsum={lsummax}-Tmx={Tmax}".format(**specs)
-        uniquefilename+="-om={omega}-th={theta}".format(**specs)
+        if specs["nnsrg"] != 'none':
+          uniquefilename+="-srgNN={lambdaSRGNN}-{potbareNN}-{ostatSRGNN}-{cutnumSRGNN}-{empotsrg}".format(**specs)
+        if specs["tnfsrg"] != 'False':
+          uniquefilename+="-srg3N={lambdaSRG3N}-{potbareNNTNF}-{ostatSRGTNF}-{cutnumSRGTNF}".format(**specs)
           
-        hashname=hashlib.sha256(str.encode(uniquefilename)).hexdigest()
-    
-        return uniquefilename,hashname
-    
-    
-    def _check_specs_(self,specs):
-        """raises a ValueError if defined specifications are not allowed"""
-
-        if not (specs["kind"] in ["one","two"]):
-            raise(ValueError("'kind' not defined"))
-        if not (specs["nninter"] in ["av18","sms","cdbonn"]):
-            raise(ValueError("'nninter' not defined"))
-        if not (specs["order"] in ["lo","nlo","n2lo","n3lo","n4lo","n4lo+"]):
-            raise(ValueError("'order' not defined"))
-        if specs["cutoff"] < 0.0:
-            raise(ValueError("'cutoff' negative"))
-        if not (specs["tnf"] in ["none","n2lo","n3lo"]):
-            raise(ValueError("'tnf' not defined"))
-        if not (specs["srg"] in ["none","2body","3body"]):
-            raise(ValueError("'srg' not defined"))
-        if specs["srgval"] < 0.0:
-            raise(ValueError("'srgval' not defined"))
-        if (specs["N"] != int(specs["N"])) or (specs["N"] < 1) :
-            raise(ValueError("'N' not defined"))
-        if (specs["Z"] != int(specs["Z"])) or (specs["Z"] < 1) :
-            raise(ValueError("'Z' not defined"))
-        
-        if (specs["Jtot"] != int(specs["Jtot"])) or (specs["Jtot"] < 0) :
-            raise(ValueError("'Jtot' not defined"))
-        if  specs["MTtot"] != int(specs["MTtot"]) :
-            raise(ValueError("'MTtot' not defined"))
-
-        if (specs["j12max"] != int(specs["j12max"])) or (specs["j12max"] < 0) :
-            raise(ValueError("'j12max' not defined"))
-        if (specs["lmax"] != int(specs["lmax"])) or (specs["lmax"] < 0) :
-            raise(ValueError("'lmax' not defined"))
-        if (specs["Tmax"] != int(specs["Tmax"])) or (specs["Tmax"] < 0) :
-            raise(ValueError("'Tmax' not defined"))
-        if (specs["Nmax"] != int(specs["Nmax"])) or (specs["Nmax"] < 0) :
-            raise(ValueError("'Nmax' not defined"))
-        if specs["OmegaHO"] < 0.0:
-            raise(ValueError("'OmegaHO' negative"))
-        if specs["omega"] < 0.0:
-            raise(ValueError("'omega' negative"))
-        
-        if (specs["theta"] < 0.0) or (specs["theta"] > 180.0):
-            raise(ValueError("'theta' out of range"))
-        
+        if specs["NP12A"]+specs["NP12B"]!=0:   
+          uniquefilename+="-np12a={NP12A}-np12b={NP12B}".format(**specs)
+        elif  specs["NP12"]!=0:
+          uniquefilename+="-np12={NP12}".format(**specs)
+          
+        if specs["NP3A"]+specs["NP3B"]!=0:   
+          uniquefilename+="-np3a={NP3A}-np3b={NP3B}".format(**specs)
+        elif  specs["NP3"]!=0:
+          uniquefilename+="-np3={NP3}".format(**specs)
+          
+        if specs["NQ4A"]+specs["NQ4B"]!=0:   
+          uniquefilename+="-nq4a={NQ4A}-nq4b={NQ4B}".format(**specs)
+        elif  specs["NQ4"]!=0:
+          uniquefilename+="-nq4={NQ4}".format(**specs)
             
+        if specs["NQA"]+specs["NQB"]!=0:   
+          uniquefilename+="-nqa={NQA}-nqb={NQB}".format(**specs)
+        elif  specs["NQ"]!=0:
+          uniquefilename+="-nq={NQ}".format(**specs)
+          
+        if specs["NPTA"]+specs["NPTB"]!=0:   
+          uniquefilename+="-npta={NPTA}-nptb={NPTB}".format(**specs)
+        elif  specs["NPT"]!=0:
+          uniquefilename+="-npt={NPT}".format(**specs)
+            
+        if specs["NEA"]+specs["NEB"]!=0:   
+          uniquefilename+="-nea={NEA}-neb={NEB}".format(**specs)
+        elif  specs["NE"]!=0:
+          uniquefilename+="-ne={NE}".format(**specs)
+        if  specs["NX"]!=0:
+          uniquefilename+="-nx={NX}".format(**specs)
+          
+            
+        uniquefilename+="-jmx={j12max}-lmx={lmax}-lsum={lsummax}-Tmx={taumax}".format(**specs)
+        # now add the additional parameters of density
+        uniquefilename+="-nxt={nxtilde}-nph={nphitilde}".format(**specs)
+        uniquefilename+="-j12mxd={j12maxdens}-lmxd={lmaxdens}-taumxd={taumaxdens}".format(**specs)
+        try:
+          uniquefilename+="-lsmxd={lsummaxdens}".format(**specs)
+        except:
+          pass
+        uniquefilename+="-om={omega}-th={theta}".format(**specs)
         
-    def get_file(self,kind="one", 
-                 nninter="sms",order="lo",cutoff=450,
-                 tnf="None",c1=0.0,c3=0.0,c4=0.0,cE1=0.0,cE2=0.0,cE3=0.0,cE4=0.0,cE5=0.0,cE6=0.0,cE7=0.0,cE8=0.0,cE9=0.0,cE10=0.0,
-                 srg="None",srgval=0.0,
-                 N=1,Z=2,Jtot=1,MTtot=1,
-                 j12max=5,lmax=6,Tmax=3,lsummax=0,
-                 Nmax=0,OmegaHO=0.0,
-                 omega=0.0,theta=0.0,**kwargs
-               ):
-        """This routine gets the local name of a file included in the database
-           kind - "one" or "two" for one- or two-body density, 
-           nninter - nn interaction used,
-           order - chiral order of interaction "lo,nlo,n2lo,...,n4lo+",
-           cutoff - numerical value of cutoff (either momentum or rspace),
-           tnf - 3NF used "None", "n2lo","n3lo",
-           c1,c3,c4 - N2LO parameters of 3NF,
-           cE1,...,cE10 - N4LO parameters of 3NF, 
-           srg - "None","2body","3body", 
-           srgval - if srg="2body" or "3body", value used for the SRG evolution (s or cutoff),
-           N - number of neutrons in nucleus,
-           Z - number of protons in nucleus,
-           Jtot  - twice total angular momentum of state,
-           MTtot - twice third component of isospin,
-           j12max - max. j12 used for calculation,
-           lmax - max. orbital angular momentum used in calc,
-           lsummax - max. sum of orbital angular momentum used in calc,
-           Tmax - twice maximal isospin used in calculation,
-           Nmax - maximal HO quantum number in NCSM if applicable,
-           OmegaHO - HO frequency used in NCSM calculation if applicable
-           omega - incoming photon energy (for compton scattering)
-           theta - photon scattering angle in deg
-
-           on exit the local filename (hashname) and a unique filename are returned.
-           if not available locally, the file is downloaded
-           if not available remote, a ValueError is raised 
+        hashname=hashlib.sha256(str.encode(str(specs))).hexdigest()
+    
+        return uniquefilename,hashname        
+        
+    def get_file(self,**labels):
+        """Labels should be given according to standard set (see in _prep_label_).
+           On exit the local filename (hashname) and a unique filename are returned.
+           If not available locally, the file is downloaded.
+           If not available remote, a ValueError is raised. 
         """
 
         # prepare dictionary specifying the case wanted
-
-        specs={"kind":kind.lower(), 
-               "nninter":nninter.lower(),
-               "order":order.lower(),
-               "cutoff":cutoff,
-               "tnf":tnf.lower(),
-               "c1":c1,"c3":c3,"c4":c4,"cE1":cE1,"cE2":cE2,"cE3":cE3,"cE4":cE4,"cE5":cE5,"cE6":cE6,"cE7":cE7,"cE8":cE8,"cE9":cE9,"cE10":cE10,
-               "srg":srg.lower(),"srgval":srgval,
-               "N":N,"Z":Z,"Jtot":Jtot,"MTtot":MTtot,
-               "j12max":j12max,"lmax":lmax,"lsummax":lsummax,"Tmax":Tmax,
-               "Nmax":Nmax,"OmegaHO":OmegaHO,
-               "omega":omega,"theta":theta}
-
+        specs=labels
         # look for case in database         
         try:
           select=(self.pddf["kind"]==specs["kind"])
           for key, value in specs.items():
-            select &= (self.pddf[key]==value) 
+            try:  # do not test for columns that are undefined 
+              usevalue=not np.isnan(value)
+            except:
+              usevalue=True    
+            if usevalue:    
+              select &= (self.pddf[key]==value)                
           if len(self.pddf.loc[select]) > 0:
              densitypresent=True
           else:   
@@ -372,11 +478,20 @@ class database:
         # if not present, raise an error
         if not densitypresent:  
             raise ValueError("Density is not present in this database!")
-
+        
+        # check for unique file
+        nhits=len(self.pddf.loc[select])
+        if nhits!=1:
+            print("Found several hits:")
+            print(self.pddf.loc[select].to_markdown())
+            raise ValueError("Wave function not unique: {0:d} hits found!".format(nhits))
+        
         # then get filenames ...
         hashname=self.pddf.loc[select]["hashname"].item()
         uniquefilename=self.pddf.loc[select]["uniquefilename"].item()
-        
+        # and keep the complete labels for later use 
+        self.lastaccess=self.pddf.loc[select].to_dict(orient='records')[0]
+                
         # if the density is present check whether it is locally available
         if not bool(self.pddf.loc[select]["localavail"].item()):
             index=int(self.pddf.index[select].tolist()[0])
@@ -539,6 +654,55 @@ class database:
        savedf=self.pddf.copy(deep=True)
        savedf.to_hdf(self.workdir+self.pddf_hdf, key='densityfiles', mode='w')
 
+    def prep_upload(self,tmpdir):
+        """gzips and moves all new files to a temporary directory.
+           Leaves the local copies unchanged.
+           Also saves table and copies this file to temp directory.
+           tmpdir -- directory for temporarily storing new files
+           This method is meant as a preparation to upload new files to
+           remote database. 
+        """
+
+        # add "/" to filename if required
+        if tmpdir!="":   
+          if tmpdir[-1]!="/":
+            tmpdir=tmpdir+"/"
+                       
+
+            # first select all new files in database
+        try:
+          select=(self.pddf["localonly"]==True)
+          if len(self.pddf.loc[select]) > 0:
+             denspresent=True
+          else:   
+             denspresent=False
+        except:
+          # assume empty frame   
+          denspresent=False
+
+        # if some new local files are present 
+        if denspresent:  
+          for index,row in self.pddf[select].iterrows():
+              # first generate a gzipped version of the file
+              hashname=row["hashname"]
+              with open(self.workdir+hashname, 'rb') as f_in:
+                with gzip.open(self.workdir+hashname+".gz", 'wb') as f_out:
+                  shutil.copyfileobj(f_in, f_out)
+
+              # now upload the new gzipped file
+              shutil.copy(self.workdir+hashname+".gz",tmpdir)
+              # remove gzipped file 
+              os.remove(self.workdir+row["hashname"]+".gz")              
+              # and mark accordingly in table 
+              self.pddf.at[index,"localonly"]=False
+
+        # the write table to disk
+        savedf=self.pddf.copy(deep=True)
+        savedf["localonly"]=False
+        savedf["localavail"]=False
+        savedf.to_hdf(self.workdir+self.pddf_hdf, key='densityfiles', mode='w')
+        # and copy table file
+        shutil.copy(self.workdir+self.pddf_hdf,tmpdir)
        
 
 class densfile2b:
@@ -776,6 +940,9 @@ class densfile1b:
         indxflag=False
         getline=False
         self.rho1b=np.empty((self.maxrho1bindex),dtype=np.double)
+        lastlinecount=self.maxrho1bindex//6+1
+        lastlinedata=self.maxrho1bindex%6
+        
         for line in filein:
           if getline:
             data=line.split()
@@ -784,9 +951,9 @@ class densfile1b:
               self.thetaval=float(data[1])
               self.qval=float(data[2])
               self.thetaqval=float(data[3])              
-            elif count==22:  
-              self.rho1b[(count-1)*6]=data[0]
-              self.rho1b[(count-1)*6+1]=data[1]
+            elif count==lastlinecount:
+              for col in range(lastlinedata):
+                self.rho1b[(count-1)*6+col]=data[col]
             else:
               self.rho1b[(count-1)*6]=data[0]
               self.rho1b[(count-1)*6+1]=data[1]
